from django.contrib.auth.models import AnonymousUser, User
from django.contrib import messages
from django.contrib.sessions.backends.base import SessionBase
from django.core.exceptions import ImproperlyConfigured
import pytest

from incuna_test_utils.testcases.request import (
    BaseRequestTestCase,
    DummyStorage,
)
from tests.factories import UserFactory


class RequestTestCase(BaseRequestTestCase):
    user_factory = UserFactory

    def test_create_request(self):
        request = self.create_request()
        assert request.method == 'GET'
        assert request.path == '/'
        assert isinstance(request.user, User)
        assert isinstance(request._messages, DummyStorage)

    def test_create_request_post(self):
        request = self.create_request('post')
        assert request.method == 'POST'

    def test_create_request_url(self):
        url = '/foo/'
        request = self.create_request(url=url)
        assert request.path == url

    def test_create_request_user(self):
        user = UserFactory.build()
        request = self.create_request(user=user)
        assert request.user == user

    def test_create_request_anonymous(self):
        request = self.create_request(auth=False)
        assert isinstance(request.user, AnonymousUser)

    def test_create_request_data(self):
        data = {'foo': 'bar'}
        request = self.create_request(data=data)
        assert request.DATA == data

    def test_create_user(self):
        user = self.create_user()
        assert isinstance(user, User)

    def test_create_user_anonymous(self):
        user = self.create_user(auth=False)
        assert isinstance(user, AnonymousUser)

    def test_add_session_to_request(self):
        request = self.create_request()
        self.add_session_to_request(request)
        assert isinstance(request.session, SessionBase)

    def test_get_view(self):
        with pytest.raises(ImproperlyConfigured):
            self.get_view()


def function_view(request):
    return request


class RequestTestCaseFunctionView(BaseRequestTestCase):
    view = function_view

    def test_get_view(self):
        view = self.get_view()
        assert view == function_view


class ClassView:
    @classmethod
    def as_view(cls):
        return function_view


class RequestTestCaseClassView(BaseRequestTestCase):
    view = ClassView

    def test_get_view(self):
        view = self.get_view()
        assert view == function_view


class TestDummyStorage:
    def test_add(self):
        message_storage = DummyStorage()

        message = "I've added a message!"
        message_storage.add(messages.INFO, message)

        assert message in message_storage.store

    def test_iter(self):
        message_storage = DummyStorage()
        message = "A message!"
        message_storage.store = [message]

        assert message in iter(message_storage)



import django


DJANGO_LT_15 = django.VERSION < (1, 5)
DJANGO_LT_16 = django.VERSION < (1, 6)
DJANGO_LT_17 = django.VERSION < (1, 7)


def wipe_id_fields_on_django_lt_17(fields):
    """
    Remove fields ending in '_id' on Django < 1.7

    This required for tests checking the list of fields on a model because in
    Django 1.7, all the FKs appear as `other_model` and `other_model_id`.
    """
    if DJANGO_LT_17:
        return [field for field in fields if not field.endswith('_id')]
    return fields


class Python2AssertMixin(object):
    """
    Add python 3 asserts to python 2 TestCase

    Asserts added:
    * assertCountEqual
    * assertRegex
    """

    def __init__(self, *args, **kwargs):
        super(Python2AssertMixin, self).__init__(*args, **kwargs)
        if not hasattr(self, 'assertCountEqual'):
            self.assertCountEqual = self.assertItemsEqual

        if not hasattr(self, 'assertRegex'):
            self.assertRegex = self.assertRegexpMatches



import factory

from feincms.module.page.models import Page


class PageFactory(factory.DjangoModelFactory):
    title = factory.Sequence('Page {0}'.format)
    slug = factory.Sequence('page-{0}'.format)

    class Meta:
        model = Page



from django.core.urlresolvers import resolve, reverse
from django.test import TestCase


class URLTestMixin(object):
    def assert_url_matches_view(self, view, expected_url, url_name,
                                url_args=None, url_kwargs=None, urlconf=None):
        """
        Assert a view's url is correctly configured

        Check the url_name reverses to give a correctly formated expected_url.
        Check the expected_url resolves to the expected view.
        """

        reversed_url = reverse(url_name, urlconf=urlconf, args=url_args, kwargs=url_kwargs)
        self.assertEqual(reversed_url, expected_url)

        resolved_view = resolve(expected_url, urlconf=urlconf).func

        if hasattr(resolved_view, 'cls'):
            self.assertEqual(resolved_view.cls, view)
        else:
            self.assertEqual(resolved_view.__name__, view.__name__)


class URLTestCase(URLTestMixin, TestCase):
    pass



from incuna_test_utils.testcases.urls import URLTestCase

from .. import views


class TestURLTestCase(URLTestCase):
    def test_assert_url_matches_view(self):
        self.assert_url_matches_view(
            view=views.my_view,
            expected_url='/',
            url_name='function-view',
        )

    def test_assert_url_matches_view_class(self):
        self.assert_url_matches_view(
            view=views.MyView,
            expected_url='/foo/',
            url_name='class-view',
        )

    def test_assert_url_matches_view_rest_framework(self):
        self.assert_url_matches_view(
            view=views.MyAPIView,
            expected_url='/bar/',
            url_name='api-view',
        )



# -*- coding: utf-8 -*-
from django.contrib.sessions.backends.base import SessionBase
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
import pytest

from incuna_test_utils.testcases.integration import BaseIntegrationTestCase
from tests.factories import UserFactory
from tests.views import my_view, MyTemplateView


TEMPLATE_CONTENT = u'†εмρʟαтℯ ℭøηт℮ηт.\n'


@pytest.fixture(scope='module')
def simple_integration():
    class IntegrationTestCase(BaseIntegrationTestCase):
        user_factory = UserFactory
        view = my_view

    # Python 2 doesn't allow instantiation of a TestCase without a
    # specified test method, so specify a method known to exist on
    # all TestCase instances. We don't care which method this is.
    return IntegrationTestCase(methodName='__init__')


@pytest.fixture(scope='module')
def template_view_integration():
    class IntegrationTestCase(BaseIntegrationTestCase):
        user_factory = UserFactory
        view = MyTemplateView

    # Python 2 doesn't allow instantiation of a TestCase without a
    # specified test method, so specify a method known to exist on
    # all TestCase instances. We don't care which method this is.
    return IntegrationTestCase(methodName='__init__')


class TestIntegration:
    @pytest.mark.django_db
    def test_access_view(self, simple_integration):
        response = simple_integration.access_view()
        assert isinstance(response, HttpResponse)

        request = response.request
        assert isinstance(request, WSGIRequest)
        assert isinstance(request.session, SessionBase)

    @pytest.mark.django_db
    def test_access_view_request(self, simple_integration):
        request = simple_integration.create_request()
        response = simple_integration.access_view(request=request)

        assert response.request == request

        with pytest.raises(AttributeError):
            response.request.session

    def test_assert_count(self, simple_integration):
        haystack = [1, 1, 3, 1, 2, 4]
        needle = 1
        count = 3
        simple_integration.assert_count(needle, haystack, count)

    def test_assert_count_fail(self, simple_integration):
        haystack = [1, 1, 3, 1, 2, 4]
        needle = 1
        count = 4

        with pytest.raises(AssertionError):
            simple_integration.assert_count(needle, haystack, count)

    def test__assert_count_message(self, simple_integration):
        haystack = [1, 1, 3, 1, 2, 4]
        needle = 1
        count = 1
        actual_count = haystack.count(needle)

        expected_message_args = (count, needle, actual_count, haystack)
        expected_message = 'Expected {} instance of {}, but found {}, in {}'
        expected_message = expected_message.format(*expected_message_args)

        message_args = (needle, haystack, count, actual_count)
        message = simple_integration._assert_count_message(*message_args)
        assert message == expected_message

    def test__assert_count_message_plural(self, simple_integration):
        haystack = [1, 1, 3, 1, 2, 4]
        needle = 1
        count = 4
        actual_count = haystack.count(needle)

        expected_message_args = (count, needle, actual_count, haystack)
        expected_message = 'Expected {} instances of {}, but found {}, in {}'
        expected_message = expected_message.format(*expected_message_args)

        message_args = (needle, haystack, count, actual_count)
        message = simple_integration._assert_count_message(*message_args)
        assert message == expected_message

    def test_render_to_str(self, template_view_integration):
        request = template_view_integration.create_request(auth=False)
        response = template_view_integration.access_view(request=request)

        content = template_view_integration.render_to_str(response)
        assert content == TEMPLATE_CONTENT

    def test_access_view_and_render_response(self, template_view_integration):
        request = template_view_integration.create_request(auth=False)

        content = template_view_integration.access_view_and_render_response(
            request=request,
        )
        assert content == TEMPLATE_CONTENT



from django.contrib.auth.models import AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase, RequestFactory


class DummyStorage:
    def __init__(self):
        self.store = list()

    def add(self, level, message, extra_tags=''):
        self.store.append(message)

    def __iter__(self):
        for item in self.store:
            yield item


class BaseRequestTestCase(TestCase):
    """
    Extend django.test.TestCase with a create_request method.

    BaseRequestTestCase must be subclassed with a user_factory attribute to
    create a default user for the request.

    A class- or function-based view can be attached to the test class as the
    'view' attribute.  get_view() returns a callable version of that
    view, abstracting over whether it's class- or function-based.
    """

    request_factory = RequestFactory

    @classmethod
    def get_view(cls):
        """
        Returns the class's attached view, as a callable.

        Checks self.view exists, and throws an ImproperlyConfigured exception
        if it doesn't.  Otherwise, it returns the view, ensuring it's callable.
        """
        try:
            view = cls.__dict__['view']
        except KeyError:
            message = "This test must have a 'view' attribute."
            raise ImproperlyConfigured(message)

        try:
            return view.as_view()
        except AttributeError:
            return view

    @staticmethod
    def add_session_to_request(request):
        """Annotate a request object with a session."""
        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()

    def create_request(self, method='get', url='/', user=None, auth=True, add_session=False, **kwargs):
        if user is None:
            user = self.create_user(auth=auth)
        request = getattr(self.request_factory(), method)(url, **kwargs)
        request.user = user

        if 'data' in kwargs:
            request.DATA = kwargs['data']

        request._messages = DummyStorage()

        if add_session:
            self.add_session_to_request(request)

        return request

    def create_user(self, auth=True, **kwargs):
        if auth:
            return self.user_factory.create(**kwargs)
        else:
            return AnonymousUser()



from rest_framework.test import APIRequestFactory, force_authenticate

from .request import BaseRequestTestCase


class BaseAPIRequestTestCase(BaseRequestTestCase):
    """
    Extend BaseRequestTestCase with REST framework compatibility.

    Must be subclassed with an user_factory attribute in order to work.
    """
    request_factory = APIRequestFactory

    def create_request(self, method='get', url='/', user=None, auth=True, **kwargs):
        kwargs['format'] = 'json'
        request = super(BaseAPIRequestTestCase, self).create_request(
            method=method,
            url=url,
            user=user,
            auth=auth,
            **kwargs)
        if auth:
            force_authenticate(request, request.user)
        return request



import pytest

from ..models import User
from incuna_test_utils.factories.user import BaseUserFactory


class UserFactory(BaseUserFactory):
    class Meta:
        model = User


@pytest.mark.django_db
def test_userfactory():
    user = UserFactory.build()
    assert isinstance(user, User)
    assert 'example.com' in user.email
    assert 'Test User' in user.name



from django.shortcuts import render

from .request import BaseRequestTestCase


class BaseIntegrationTestCase(BaseRequestTestCase):
    """
    A TestCase that operates similarly to a Selenium test.

    Contains methods that access pages and render them to strings full of
    HTML.  Can be used to assert the contents of templates as well as doing
    normal TestCase things.

    Must be subclassed with the following attributes in order to work:
    * user_factory
    * view (class-based or function-based view)
    """

    def access_view(self, *args, **kwargs):
        """
        Helper method that accesses the test's view.

        Accepts an optional 'request' kwarg.  If this isn't supplied,
        access_view creates a basic request on your behalf.

        Returns a HTTPResponse object with the request (created or otherwise)
        attached.
        """
        request = kwargs.pop('request', None)
        if request is None:
            request = self.create_request(add_session=True)

        view_callable = self.get_view()
        response = view_callable(request, *args, **kwargs)

        # Add the request to the response.
        # This is a weird-looking but compact way of ensuring we have access to
        # the request everywhere we need it, without doing clunky things like
        # returning tuples all the time.
        response.request = request
        return response

    def render_to_str(self, response, request=None):
        """
        Render a HTTPResponse into a string that holds the HTML content.

        Accepts an optional request parameter, and looks for a request attached
        to the response if the optional parameter isn't specified.
        """
        if request is None:
            request = response.request

        response = render(request, response.template_name, response.context_data)
        return response.content.decode('utf-8')

    def access_view_and_render_response(self, *args, **kwargs):
        """
        Accesses the view and returns a string of HTML.

        Combines access_view, an assertion on the returned status, and
        render_to_str.

        Accepts an optional 'request' kwarg holding a HTTPRequest, but will
        create a simple one if the parameter isn't supplied, and
        'expected_status', an expected status code for the response, which
        defaults to 200.  Other args and kwargs are passed on to the view
        method.
        """
        request = kwargs.pop('request', None)
        expected_status = kwargs.pop('expected_status', 200)

        response = self.access_view(*args, request=request, **kwargs)

        # Assert that the response has the correct status code before we go
        # any further.  Throwing accurately descriptive failures when something
        # goes wrong is better than trying to run assertions on the content
        # of a HTML response for some random 404 page.
        self.assertEqual(expected_status, response.status_code)

        # Render the response and return it.
        return self.render_to_str(response)

    @staticmethod
    def _assert_count_message(needle, haystack, count, actual_count):
        # Build a verbose error message in case we need it.
        plural = '' if count == 1 else 's'
        message = 'Expected {count} instance{plural} of {needle}, but found {actual_count}, in {haystack}'
        return message.format(
            count=count,
            plural=plural,
            needle=needle,
            actual_count=actual_count,
            haystack=haystack,
        )

    def assert_count(self, needle, haystack, count):
        """
        Assert that 'needle' occurs exactly 'count' times in 'haystack'.

        Used as a snazzier, stricter version of unittest.assertIn.
        Outputs a verbose error message when it fails.
        """
        actual_count = haystack.count(needle)

        message_args = (needle, haystack, count, actual_count)
        message = self._assert_count_message(*message_args)

        self.assertEqual(count, actual_count, message)



import dj_database_url


DATABASES = {
    'default': dj_database_url.config(
        default='postgres://localhost/test_utils',
    ),
}
INSTALLED_APPS = (
    'tests',

    'feincms.module.page',

    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
)
MIDDLEWARE_CLASSES = ()
PASSWORD_HASHERS = ['django.contrib.auth.hashers.MD5PasswordHasher']
ROOT_URLCONF = 'tests.urls'
SECRET_KEY = 'test'



from django.conf.urls import url

from . import views


urlpatterns = [
    url('foo/', views.MyView.as_view(), name='class-view'),
    url('bar/', views.MyAPIView.as_view(), name='api-view'),
    url('', views.my_view, name='function-view'),
]



from setuptools import setup, find_packages


setup(
    name='incuna-test-utils',
    packages=find_packages(),
    include_package_data=True,
    version='5.0.0',
    description='Custom TestCases and other test helpers for Django apps',
    long_description=open('README.md').read(),
    author='Incuna',
    author_email='admin@incuna.com',
    url='https://github.com/incuna/incuna-test-utils/',
    install_requires=[],
    zip_safe=False,
)



from django.http import HttpResponse
from django.views.generic import TemplateView, View
from rest_framework.response import Response
from rest_framework.views import APIView


def my_view(request, *args, **kwargs):
    return HttpResponse('My page.')


class MyView(View):
    def get(self, *args, **kwargs):
        return HttpResponse('My page too.')


class MyAPIView(APIView):
    def get(self, request, format=None):
        return Response({})


class MyTemplateView(TemplateView):
    template_name = 'page.html'



from feincms.module.page.models import Page
import pytest

from incuna_test_utils.factories.feincms_page import PageFactory


@pytest.mark.django_db
def test_pagefactory():
    page = PageFactory.build()
    assert isinstance(page, Page)
    assert 'Page' in page.title
    assert 'page' in page.slug



from mock import patch
from unittest import TestCase

import pytest

from incuna_test_utils import compat


def test_wipe_id_fields_lt_17():
    fields = ['foo', 'foo_id']
    expected_fields = ['foo']

    with patch.object(compat, 'DJANGO_LT_17', new=True):
        wiped_fields = compat.wipe_id_fields_on_django_lt_17(fields)
        assert wiped_fields == expected_fields


def test_wipe_id_fields_gte_17():
    fields = ['foo', 'foo_id']

    with patch.object(compat, 'DJANGO_LT_17', new=False):
        wiped_fields = compat.wipe_id_fields_on_django_lt_17(fields)
        assert wiped_fields == fields


@pytest.fixture(scope='module')
def testcase():
    """Return a TestCase instance with python 3 assert methods."""
    class Python2AssertTestCase(compat.Python2AssertMixin, TestCase):
        """A TestCase with python 3 asserts available."""

    # Python 2 doesn't allow instantiation of a TestCase without a
    # specified test method, so specify a method known to exist on
    # all TestCase instances. We don't care which method this is.
    return Python2AssertTestCase(methodName='__init__')


def test_assert_count_equal(testcase):
    """
    Check assertCountEqual is available on a TestCase with Python2AssertMixin.
    """
    testcase.assertCountEqual((1, 1, 2), (1, 2, 1))

    with pytest.raises(AssertionError):
        testcase.assertCountEqual((1, 1, 2), (2, 1, 2))


def test_assert_regex(testcase):
    """
    Check assertRegex is available on a TestCase with Python2AssertMixin.
    """
    testcase.assertRegex('foo-bar', '[a-z]+')

    with pytest.raises(AssertionError):
        testcase.assertRegex('foo-bar', '[A-Z]+')



from incuna_test_utils.testcases.api_request import BaseAPIRequestTestCase
from tests.factories import UserFactory


class APIRequestTestCase(BaseAPIRequestTestCase):
    user_factory = UserFactory

    def test_create_request_format(self):
        request = self.create_request()
        assert request.META['format'] == 'json'

    def test_create_request_auth(self):
        request = self.create_request()
        assert request.user.is_authenticated()

    def test_create_request_no_auth(self):
        request = self.create_request(auth=False)
        assert not request.user.is_authenticated()
